/**
 * File:
 *  fader.h
 *
 * Notes:
 *  This file contains the type definitions and prototypes for my own fader
 *  routines
 *
 * History:
 *  17/01/2013 Written by Simon Platten
 */
#ifndef FADER_H
  #define FADER_H

  #include "types.h"
// Constants
  #define FADER_START_PT    100
  #define FADE_DELTA        1     // 1 = Slowest

  typedef enum {
    FADE_IN = 0,
    FADE_OUT
  } eFadeDirections;

  typedef enum {
    SINGLE_FADER = 0,
    MULTIPLE_FADERS
  } eFadeMode;

  typedef struct _faderNode {
    unsigned          uBounce : 1; // 1 = Bounce back from current state
    unsigned          uDir    : 1; // 0 = Fade In, 1 = Fade Out
    unsigned          uEnable : 1; // 0 = Disabled, 1 = Enabled
    unsigned          uLoop   : 1; // 1 = Loop constantly
    unsigned          uMode   : 1; // 0 = Single Fader, 1 = Multiple Faders
    unsigned          uState  : 1; // 0 = Off, 1 = On
    ushort            *pPort;      // The port the LED is attached to
    byte              bytIOmask;   // The pins to effect, 1, 2, 4, 8, 16 etc.
    byte              bytBounces;  // The bounce count
    byte              bytDelta;    // Rate of change delta / speed, 1 = Slowest
    uint              uintCount;   // % of duty cycle or off cycle
    uint              uintDutyTime;// % to be on
    int               intOffTime;  // % to be off
    uint              uintRepeats; // The number of repeats per fade iteration
    uint              uintRptCtr;  // The current repeat counter
  } faderDef;
// Function prototypes
  boolean initFader(faderDef        *pF,
                    ushort          *pPort,
                    byte            bytIOmask,
                    eFadeMode       eMode,
                    eFadeDirections eDir,
                    boolean         blnLoop,
                    boolean         blnBounce,
                    boolean         blnEnable,
                    byte            bytDelta,
                    uint            uintRepeats);
  void serviceFaders(void);
#endif